﻿/*
 * This file contains the core of the extension's implementation, including the methods that are
 * exposed to Fiddler to call (e.g. the implementation of the IAutoTamper2 interface).
*/
using System;
using System.Collections.Generic;
using System.Text;
using Fiddler;
using System.Reflection;
using System.Windows.Forms;
using Microsoft.Win32;

// You MUST indicate the version of Fiddler your extension requires. Fail to do so,
// and Fiddler will not load your extension.
[assembly: RequiredVersion("2.4.9.0")]
namespace AnyWhere
{
    public class GoAnywhere : IAutoTamper2
    {
        /// <summary>
        /// The UI that this extension uses
        /// </summary>
        private frmSelectLocation oFrmLocation = null;

        // Menu items we'll be adding to the main Fiddler menu
        private MenuItem mnuTravel;
        private MenuItem miLookup;
        private MenuItem miSetLocation;
   
        /// <summary>
        /// Flag indicating whether location override is enabled. It's a best practice to have
        /// such a boolean rather than doing expensive calculations inside the Fiddler event handlers.
        /// </summary>
        internal bool bEnabled = false;

        // Location-holding Variables
        private string _sLongitude;
        private string _sLatitude;
        private string _sAltitude;
        private string _sPrecision;

        #region IAutoTamper2 Implementation
        public void AutoTamperRequestAfter(Session oSession)
        {
            // This method runs just before the request is sent to the server,
            // just after the user has the opportunity to tamper with the request.
        }

        public void AutoTamperRequestBefore(Session oSession)
        {
            // This method runs just after Fiddler reads the complete request from
            // the client. It runs just before the user has the opportunity to tamper
            // with the request.
            if (!bEnabled) return;

            // Old Firefox
            if (oSession.fullUrl.Equals("https://www.google.com/loc/json", StringComparison.OrdinalIgnoreCase))
            {
                string sGOOGTemplate = "{{\"location\":{{\"latitude\":{0},\"longitude\":{1},\"accuracy\":{2}}},\"access_token\":\"2:p7865:9AjuzHoP_RX\"}}";
                string sLocationJSON = String.Format(sGOOGTemplate,
                    _sLongitude, _sLatitude, _sPrecision);
                oSession.utilCreateResponseAndBypassServer();
                oSession.utilSetResponseBody(sLocationJSON);
                oSession.oResponse.headers["Content-Type"] = "application/json; charset=UTF-8";
                oSession.oResponse.headers["Cache-Control"] = "max-age=1;";
                oSession["ui-backcolor"] = "#99FFCC";
                return;
            }

            // New Firefox
            if (oSession.fullUrl.StartsWith("https://location.services.mozilla.com/v1/geolocate?key=", StringComparison.OrdinalIgnoreCase))
            {
                string sMozillaTemplate = "{{\"location\": {{\"lat\": {0}, \"lng\": {1}}}, \"accuracy\": {2}}}";
                string sLocationJSON = String.Format(sMozillaTemplate, _sLongitude, _sLatitude, _sPrecision);
                oSession.utilCreateResponseAndBypassServer();
                oSession.utilSetResponseBody(sLocationJSON);
                oSession.oResponse.headers["Content-Type"] = "application/json; charset=UTF-8";
                oSession.oResponse.headers["Cache-Control"] = "max-age=1;";
                oSession["ui-backcolor"] = "#99FFCC";
                return;
            }

            if (oSession.fullUrl.Equals("https://inference.location.live.net/inferenceservice/v21/Pox/GetLocationUsingFingerprint", StringComparison.OrdinalIgnoreCase))
            {
                oSession.utilCreateResponseAndBypassServer();
                string sMSFTTemplate = "<?xml version=\"1.0\" encoding=\"utf-8\"?><GetLocationUsingFingerprintResponse xmlns=\"http://inference.location.live.com\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"><GetLocationUsingFingerprintResult><ResponseStatus>Success</ResponseStatus><LocationResult><ResolverStatus Status=\"Success\" Source=\"IP\"/><ResolvedPosition Latitude=\"{0}\" Longitude=\"{1}\" Altitude=\"{3}\"/><RadialUncertainty>{2}</RadialUncertainty><TileResult/><TrackingId>a12c1545-a831-4630-829a-d12fb585578f</TrackingId></LocationResult><ExtendedV21Result CrowdSourcingLevel=\"Medium\" ServerUtcTime=\"2011-04-08T16:30:13.8141943Z\"/></GetLocationUsingFingerprintResult></GetLocationUsingFingerprintResponse>";
                string sLocationXML = String.Format(sMSFTTemplate,
                    _sLongitude, _sLatitude, _sPrecision, _sAltitude);
                oSession.utilSetResponseBody(sLocationXML);
                oSession.oResponse.headers["Content-Type"] = "application/xml; charset=utf-8";
                oSession.oResponse.headers["Cache-Control"] = "max-age=1;";
                oSession["ui-backcolor"] = "#99FFCC";
                return;
            }
        }
        
        public void OnPeekAtResponseHeaders(Session oSession)
        {
            // This method runs when response headers are available.
            // Note: The response body is not available for manipulation.
        }

        public void AutoTamperResponseBefore(Session oSession)
        { 
            // This method runs just after the response is done being read from the
            // server, just before the user has the opportunity to tamper with the response.
            //
            // NOTE: If Streaming mode is enabled for this response, this method
            // will run *after* the client gets the response.
        }

        public void AutoTamperResponseAfter(Session oSession)
        {
            // This method runs just before the final response is sent to the client
            // just after the user has the opportunity to tamper with the response.
            //
            // NOTE: If Streaming mode is enabled for this response, this method
            // will not be called!
        }

        public void OnBeforeReturningError(Session oSession)
        {
            // This method runs just before a Fiddler-generated error page (e.g. DNS Failure)
            // is returned to the client. You can modify the response in this method.
        }

        public void OnLoad()
        {
            // Called when Fiddler loads this extension.
            this.mnuTravel = new MenuItem();
            this.miLookup = new MenuItem("&Find Places");
            this.miSetLocation = new MenuItem("&Set Location...");

            mnuTravel.Text = "Any&WHERE";
            mnuTravel.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                miSetLocation, new MenuItem("-"), miLookup});

            miSetLocation.Click += new EventHandler(miSetLocation_Click);
            miLookup.Click += new EventHandler(miLookup_Click);
            FiddlerApplication.UI.Menu.MenuItems.Add(mnuTravel);
        }

        public void OnBeforeUnload()
        {
            // Called just before the Fiddler Extension is unloaded, you may perform cleanup here.
            // Currently, extensions are typically only unloaded when Fiddler is shutting down.
            //
            if (bEnabled) ClearIEGeoLocCache();
            if (null != oFrmLocation) oFrmLocation.Dispose();
        }
        #endregion IAutoTamper2 Implementation

        #region AnyWhere UI Code
        void miLookup_Click(object sender, EventArgs e)
        {
            Utilities.LaunchHyperlink("http://universimmedia.pagesperso-orange.fr/geo/loc.htm");

            // Alternative is "http://www.findlatitudeandlongitude.com" but that had a malicious advertisement once 
            // so I cannot recommend it.
        }

        void miSetLocation_Click(object sender, EventArgs e)
        {
            if (null == oFrmLocation)
            {
                oFrmLocation = new frmSelectLocation(this);
            }
            if (!oFrmLocation.Visible)
            {
                oFrmLocation.Show(FiddlerApplication.UI);
            }
            else
            {
                oFrmLocation.BringToFront();
            }
        }
        #endregion AnyWhere UI Code

        #region AnyWhere Location-Calculation-Code
        internal bool SetOverrideLocation(string sLocation)
        {
            if (String.IsNullOrEmpty(sLocation))
            {
                bEnabled = false;
                ClearIEGeoLocCache();
                FiddlerApplication.Log.LogString("/[AnyWhere] Location override is now disabled");
                return true;
            }
            bEnabled = true;
            _sLongitude = "0";
            _sLatitude = "0";
            _sAltitude = "0";
            _sPrecision = "0";

            // Strip any friendly name
            sLocation = Utilities.TrimBefore(sLocation, ":");

            // Parse the string.

            string[] sTokens = sLocation.Split(new char[] { ' ', '\t', ',' }, 4, StringSplitOptions.RemoveEmptyEntries);
            if (sTokens.Length > 1)
            {
                _sLongitude = sTokens[0];
                _sLatitude = sTokens[1];
                if (sTokens.Length > 2) _sPrecision = sTokens[2];
                if (sTokens.Length > 3) _sAltitude = sTokens[3];
                ClearIEGeoLocCache();
                FiddlerApplication.Log.LogFormat("/[AnyWhere] Location override is now enabled; set to {0}", sLocation);
                bEnabled = true;
                return true;
            }

            // TODO: Create a richer parser, especially one that can parse non-decimal (Degrees minutes seconds) format.
            //http://www.fcc.gov/mb/audio/bickel/DDDMMSS-decimal.html
            //Decimal value = Degrees + (Minutes/60) + (Seconds/3600)  http://support.microsoft.com/kb/244951

            return false;
        }

        /// <summary>
        /// Internet Explorer 9 keeps a registry cache of recent lookups to improve performance. We'll clear
        /// that cache to avoid spoiling our fun. (Another alternative would be to run IE InPrivate or use
        /// DeleteBrowserHistory, but that's a bit more work.
        /// </summary>
        private void ClearIEGeoLocCache()
        {
            try
            {
                RegistryKey oRK = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Internet Explorer\\Geolocation", true);
                if (oRK.SubKeyCount > 0)
                {
                    oRK.DeleteSubKeyTree("CacheRoot");
                }
            }
            catch (Exception eX)
            {
                FiddlerApplication.Log.LogFormat("![AnyWhere] Unable to clear IEGeoLoc cache: {0}", eX.Message);
            }
        }

        #endregion AnyWhere Location-Calculation-Code
    }
}
